﻿Imports System.Runtime.CompilerServices

Namespace System

   Public Module BindingSourceX

      ''' <summary>
      ''' provides a placeholder-syntax for filters. Sample: bs.FilterX("Datum >= ? And ? >= Datum", dtpVon.Value, dtpBis.Value)
      ''' </summary>
      ''' <exception cref="ArgumentException">expression contains more placeholders than values are passed</exception>
      <Extension()> _
      Public Sub FilterX(ByVal bs As BindingSource, ByVal expression As String, ByVal ParamArray values() As Object)
         Dim splits = expression.Split("?"c)      ' identify placeholder '?'
         If splits.Count > values.Length + 1 Then Throw New ArgumentException( _
            "expression contains more placeholders than values are passed")
         For i = 0 To splits.Length - 2
            Dim val = values(i)
            Dim format = If(TypeOf val Is String, "'{0}'", If(TypeOf val Is Date, "#{0}#", "{0}"))
            splits(i) &= String.Format(Globalization.CultureInfo.InvariantCulture, format, val)
         Next
         bs.Filter = String.Concat(splits)
      End Sub

      ''' <summary> returnt die typisierte Datarow an aktueller Position - oder Nothing. </summary>
      <Extension()> _
      Public Function At(Of T As DataRow)(ByVal subj As BindingSource) As T
         Return DirectCast(subj.At(subj.Position), T)
      End Function

      ''' <summary>
      ''' returnt die typisierte Datarow am index. Bei ungültigem index Nothing (keine OutOfRange-Exception!)
      ''' </summary>
      <Extension()> _
      Public Function At(Of T As DataRow)(ByVal subj As BindingSource, ByVal index As Integer) As T
         Return DirectCast(subj.At(index), T)
      End Function

      ''' <summary>
      ''' returnt die Datarow am index. Bei ungültigem index Nothing (keine OutOfRange-Exception!)
      ''' </summary>
      <Extension()> _
      Public Function At(ByVal subj As BindingSource, ByVal index As Integer) As DataRow
         If index < 0 OrElse index >= subj.Count Then Return Nothing
         Return DirectCast(subj(index), DataRowView).Row
      End Function

   End Module

End Namespace
