﻿Imports System.IO

Namespace System.Windows.Forms

   Public Class DataSetSaver

      Private WithEvents _Form As Form
      Public ReadOnly _DataFile As FileInfo

      Public Sub New(ByVal Form As Form, ByVal DataFile As String)
         _Form = Form
         _DataFile = New FileInfo(DataFile)
      End Sub

      Public Sub Load(ByVal dataset As DataSet)
         dataset.Clear()
         If _DataFile.Exists() Then
            For Each tb As DataTable In dataset.Tables
               tb.BeginLoadData()
            Next
            dataset.ReadXml(_DataFile.FullName)
            For Each tb As DataTable In dataset.Tables
               tb.EndLoadData()
            Next
         Else
            MsgBox(String.Concat( _
               "Leider (noch) kein DatenFile vorhanden", ControlChars.Lf, _
               "Sie können aber trotzdem fortfahren, und eines anlegen."))
         End If
         dataset.AcceptChanges()
      End Sub

      Public Sub Save(ByVal dataset As DataSet, Optional ByVal checkForChanges As Boolean = True)
         If Not _Form.Validate Then Return
         Try
            With dataset
               If checkForChanges AndAlso Not .HasChanges() Then Return
               .WriteXml(_DataFile.FullName)
               .AcceptChanges()
            End With
         Finally
            Media.SystemSounds.Asterisk.Play()
         End Try
      End Sub

      ''' <summary>returns False on Validation-Failure or Users-Abort</summary>
      Public Sub HandleFormClosing(ByVal dataset As DataSet, ByVal e As FormClosingEventArgs)
         If e.Cancel Then Return
         If Not dataset.HasChanges Then Return
         Select Case MessageBox.Show(_Form, "Änderungen speichern?", "Das Ende ist nahe", MessageBoxButtons.YesNoCancel)
            Case Windows.Forms.DialogResult.Yes
               Save(dataset, False)
            Case Windows.Forms.DialogResult.Cancel
               e.Cancel = True
               Return
         End Select
         e.Cancel = False
      End Sub

      Private Sub _Form_FormClosed(ByVal sender As Object, ByVal e As FormClosedEventArgs) Handles _Form.FormClosed
         Dim Controls = _Form.Controls
         For i = Controls.Count - 1 To 0 Step -1
            Controls(i).Dispose()
         Next
      End Sub
   End Class

End Namespace
