﻿Imports System.Runtime.CompilerServices


Namespace System

   Public Module IComparableX

      <Extension()> _
      Public Sub Maximize(Of T As IComparable)(ByRef subj As T, ByVal other As T)
         If other.CompareTo(subj) > 0 Then subj = other
      End Sub

      <Extension()> _
      Public Sub Minimize(Of T As IComparable)(ByRef subj As T, ByVal other As T)
         If other.CompareTo(subj) < 0 Then subj = other
      End Sub

      <Extension()> _
      Public Function IsBetween(Of T As IComparable)( _
            ByVal ToTest As T, _
            ByVal Bord0 As T, _
            ByVal Bord1 As T) As Boolean
         Return Bord0.CompareTo(ToTest) <= 0 AndAlso ToTest.CompareTo(Bord1) <= 0
      End Function

      <Extension()> _
      Public Function IsBetween(Of T As IComparable)( _
            ByVal ToTest As T, _
            ByVal Bord0 As T, _
            ByVal Bord1 As T, _
            ByVal AutoSort As Boolean, _
            ByVal LBoundInclude As Boolean, _
            ByVal UBoundInclude As Boolean) As Boolean
         If AutoSort AndAlso Bord0.CompareTo(Bord1) > 0 Then
            Dim Temp = Bord0
            Bord0 = Bord1
            Bord1 = Temp
         End If
         Dim C0 As Integer = Bord0.CompareTo(ToTest)
         Dim C1 As Integer = ToTest.CompareTo(Bord1)
         If LBoundInclude AndAlso C0 = 0 Then Return True
         If UBoundInclude AndAlso C1 = 0 Then Return True
         Return C0 < 0 AndAlso C1 < 0
      End Function

      <Extension()> _
      Public Function Intersect(Of T As IComparable)(ByVal Range0() As T, ByVal Range1() As T) As T()
         Dim RetVal = New T() {If(Range0(0).CompareTo(Range1(0)) > 0, Range0(0), Range1(0)), _
                               If(Range0(1).CompareTo(Range1(1)) < 0, Range0(1), Range1(1))}
         If RetVal(0).CompareTo(RetVal(1)) > 0 Then Return Nothing
         Return RetVal
      End Function

      <Extension()> _
      Public Function ClipIn(Of T As IComparable)( _
            ByVal Subj As T, ByVal LBound As T, ByVal UBound As T) As T
         If Subj.CompareTo(LBound) < 0 Then Return LBound
         If Subj.CompareTo(UBound) > 0 Then Return UBound
         Return Subj
      End Function

      <Extension()> _
      Public Function Min(Of T As IComparable)(ByVal subj As T, ByVal other As T) As T
         If subj.CompareTo(other) < 0 Then Return subj
         Return other
      End Function
      <Extension()> _
      Public Function Max(Of T As IComparable)(ByVal subj As T, ByVal other As T) As T
         If subj.CompareTo(other) > 0 Then Return subj
         Return other
      End Function

      <Extension()> _
      Public Function BinarySearch(Of T)( _
              ByVal Subj As List(Of T), _
              ByVal Pattern As T, _
              ByVal Comparison As Comparison(Of T)) As Integer
         'ermöglicht, List(Of T).BinarySearch mit Comparisons aufzurufen, statt umständlich eine IComparer-Klasse implementieren zu müssen
         Return Subj.BinarySearch(Pattern, Comparison.ToComparer)
      End Function
      <Extension()> _
      Public Function BinarySearch(Of T)( _
              ByVal Subj As T(), _
              ByVal Pattern As T, _
              ByVal Comparison As Comparison(Of T)) As Integer
         'Umbau von Shared Array.BinarySearch in eine Objektfunktion, die außerdem Comparisons akzeptiert
         Return Array.BinarySearch(Subj, Pattern, Comparison.ToComparer)
      End Function

      ''' <summary>
      ''' konvertiert eine Comparison in ein IComparer-implementierendes Objekt
      ''' </summary>
      ''' <remarks>selbst einen Delegaten kann man also erweitern</remarks>
      <Extension()> _
      Public Function ToComparer(Of T)(ByVal Subj As Comparison(Of T)) As ComparisonComparer(Of T)
         Return New ComparisonComparer(Of T)(Subj)
      End Function

      ''' <summary>IComparer-implementierender Wrapper um eine Comparison </summary>
      Public Class ComparisonComparer(Of T) : Inherits Comparer(Of T)
         Private _Comparison As Comparison(Of T)
         Public Sub New(ByVal Comparison As Comparison(Of T))
            _Comparison = Comparison
         End Sub
         Public Overrides Function Compare(ByVal x As T, ByVal y As T) As Integer
            Return _Comparison(x, y)
         End Function
      End Class

      Public Structure MinMax(Of T, T2 As IComparable)
         Public Min, Max As T
         Public MinVal, MaxVal As T2
         Private _Selector As Func(Of T, T2)
         Public ReadOnly HasValue As Boolean
         Public Sub New(ByVal first As T, ByVal selector As Func(Of T, T2))
            _Selector = selector
            Me.Min = first : Me.Max = first
            MinVal = _Selector(first) : MaxVal = MinVal
            HasValue = True
         End Sub
         Public Sub Check(ByVal itm As T)
            Dim val = _Selector(itm)
            If val.CompareTo(MinVal) < 0 Then
               Min = itm
               MinVal = val
            ElseIf val.CompareTo(MaxVal) > 0 Then
               Max = itm
               MaxVal = val
            End If
         End Sub
      End Structure


      <Extension()> _
      Public Function Extremum(Of T, T2 As IComparable)( _
            ByVal Subj As IEnumerable(Of T), ByVal selector As Func(Of T, T2)) As MinMax(Of T, T2)
         With Subj.GetEnumerator
            If Not .MoveNext Then .Dispose() : Return Nothing
            Extremum = New MinMax(Of T, T2)(.Current, selector)
            While .MoveNext : Extremum.Check(.Current) : End While
            .Dispose()
         End With
      End Function
   End Module

End Namespace
